@@ -16,23 +16,23 @@ Readonly my $DEFAULT_ARROW_TYPE =>
 	[
 	#name: $start, $body, $connection, $body_2, $end
 	
-	['origin', '', '*', '', '', '', 1],
-	['up', '|', '|', '', '', '^', 1],
-	['down', '|', '|', '', '', 'v', 1],
-	['left', '-', '-', '', '', '<', 1],
-	['upleft', '|', '|', '.', '-', '<', 1],
-	['leftup', '-', '-', '\'', '|', '^', 1],
-	['downleft', '|', '|', '\'', '-', '<', 1],
-	['leftdown', '-', '-', '.', '|', 'v', 1],
-	['right', '-', '-','', '', '>', 1],
-	['upright', '|', '|', '.', '-', '>', 1],
-	['rightup', '-', '-', '\'', '|', '^', 1],
-	['downright', '|', '|', '\'', '-', '>', 1],
-	['rightdown', '-', '-', '.', '|', 'v', 1],
-	['45', '/', '/', '', '', '^', 1, ],
-	['135', '\\', '\\', '', '', 'v', 1, ],
-	['225', '/', '/', '', '', 'v', 1, ],
-	['315', '\\', '\\', '', '', '^', 1, ],
+	['origin',       '',  '*',   '',  '',  '', 1],
+	['up',          '|',  '|',   '',  '', '^', 1],
+	['down',        '|',  '|',   '',  '', 'v', 1],
+	['left',        '-',  '-',   '',  '', '<', 1],
+	['up-left',     '|',  '|',  '.', '-', '<', 1],
+	['left-up',     '-',  '-', '\'', '|', '^', 1],
+	['down-left',   '|',  '|', '\'', '-', '<', 1],
+	['left-down',   '-',  '-',  '.', '|', 'v', 1],
+	['right',       '-',  '-',   '',  '', '>', 1],
+	['up-right',    '|',  '|',  '.', '-', '>', 1],
+	['right-up',    '-',  '-', '\'', '|', '^', 1],
+	['down-right',  '|',  '|', '\'', '-', '>', 1],
+	['right-down',  '-',  '-',  '.', '|', 'v', 1],
+	['45',          '/',  '/',   '',  '', '^', 1],
+	['135',        '\\', '\\',   '',  '', 'v', 1],
+	['225',        ' /',  '/',   '',  '', 'v', 1],
+	['315',        '\\', '\\',   '',  '', '^', 1],
 	] ;
 
 sub new
@@ -40,7 +40,7 @@ sub new
 my ($class, $element_definition) = @_ ;
 
 my $self = bless  {}, __PACKAGE__ ;
-	
+
 $self->setup
 	(
 	$element_definition->{ARROW_TYPE} || Clone::clone($DEFAULT_ARROW_TYPE),
@@ -94,7 +94,7 @@ my %direction_to_arrow =
 	'down-right' => \&draw_downright,
 	'right-down' => \&draw_rightdown,
 	) ;
-	
+
 sub get_arrow
 {
 my ($arrow_type, $end_x, $end_y, $direction, $allow_diagonal_lines) = @_ ;
@@ -113,12 +113,14 @@ my @position_to_direction =
 	) ;
 
 $direction = $position_to_direction
-			[$end_x == 0 ? CENTER : $end_x < 0 ? LEFT : RIGHT]
-			[$end_y == 0 ? CENTER : $end_y < 0 ? UP : DOWN] ;
+		[$end_x == 0 ? CENTER : $end_x < 0 ? LEFT : RIGHT]
+		[$end_y == 0 ? CENTER : $end_y < 0 ? UP : DOWN] ;
 
 return($direction_to_arrow{$direction}->($arrow_type, $end_x, $end_y, $allow_diagonal_lines), $direction) ;
 }
 
+#-----------------------------------------------------------------------------
+
 sub draw_down
 {
 my ($arrow_type, $end_x, $end_y) = @_ ;
@@ -133,11 +135,13 @@ push @{$stripes},
 	'WIDTH' => 1,
 	'X_OFFSET' => 0,
 	'Y_OFFSET' => 0,
-	};
-	
+	} ;
+
 return($stripes, $width, $height) ;
 }
 
+#-----------------------------------------------------------------------------
+
 sub draw_origin
 {
 my ($arrow_type, $end_x, $end_y) = @_ ;
@@ -152,11 +156,13 @@ push @{$stripes},
 	'WIDTH' => 1,
 	'X_OFFSET' => 0,
 	'Y_OFFSET' => 0,
-	};
-	
+	} ;
+
 return($stripes, $width, $height) ;
 } 
 
+#-----------------------------------------------------------------------------
+
 sub draw_up
 {
 my ($arrow_type, $end_x, $end_y) = @_ ;
@@ -171,11 +177,13 @@ push @{$stripes},
 	'WIDTH' => 1,
 	'X_OFFSET' => 0,
 	'Y_OFFSET' => $end_y, 
-	};
-	
+	} ;
+
 return($stripes, $width, $height) ;
 }
 
+#-----------------------------------------------------------------------------
+
 sub draw_left
 {
 my ($arrow_type, $end_x, $end_y) = @_ ;
@@ -190,11 +198,13 @@ push @{$stripes},
 	'WIDTH' => $width,
 	'X_OFFSET' => $end_x,
 	'Y_OFFSET' => 0,
-	};
-	
+	} ;
+
 return($stripes, $width, $height) ;
 }
 
+#-----------------------------------------------------------------------------
+
 sub draw_upleft # or 315
 {
 my ($arrow_type, $end_x, $end_y, $allow_diagonal_lines) = @_ ;
@@ -223,12 +233,14 @@ else
 		'WIDTH' => $width - 1,
 		'X_OFFSET' => $end_x,
 		'Y_OFFSET' => $end_y ,
-		};
+		} ;
 	}
-	
+
 return($stripes, $width, $height) ;
 }
 
+#-----------------------------------------------------------------------------
+
 sub draw_leftup # or 315
 {
 my ($arrow_type, $end_x, $end_y, $allow_diagonal_lines) = @_ ;
@@ -257,12 +269,14 @@ else
 		'WIDTH' => 1,
 		'X_OFFSET' => $end_x,
 		'Y_OFFSET' => $end_y ,
-		};
+		} ;
 	}
-	
+
 return($stripes, $width, $height) ;
 }
 
+#-----------------------------------------------------------------------------
+
 sub get_315_stripes
 {
 my ($position, $start, $body, $end) = @_ ;
@@ -276,7 +290,7 @@ push @stripes,
 	'WIDTH' => 1,
 	'X_OFFSET' => 0,
 	'Y_OFFSET' => 0,
-	};
+	} ;
 	
 for(my $xy = -$position - 1 ; $xy> 0 ; $xy--)
 	{
@@ -287,9 +301,9 @@ for(my $xy = -$position - 1 ; $xy> 0 ; $xy--)
 		'WIDTH' => 1,
 		'X_OFFSET' => -$xy,
 		'Y_OFFSET' => -$xy,
-		};
+		} ;
 	}
-	
+
 push @stripes,
 	{
 	'HEIGHT' => 1,
@@ -297,11 +311,13 @@ push @stripes,
 	'WIDTH' => 1,
 	'X_OFFSET' => $position ,
 	'Y_OFFSET' => $position ,
-	};
-	
+	} ;
+
 return(@stripes) ;
 }
 
+#-----------------------------------------------------------------------------
+
 sub draw_downleft # or 225
 {
 my ($arrow_type, $end_x, $end_y, $allow_diagonal_lines) = @_ ;
@@ -330,12 +346,14 @@ else
 		'WIDTH' => $width - 1,
 		'X_OFFSET' => $end_x,
 		'Y_OFFSET' => $end_y ,
-		};
+		} ;
 	}
-	
+
 return($stripes, $width, $height) ;
 }
 
+#-----------------------------------------------------------------------------
+
 sub draw_leftdown # or 225
 {
 my ($arrow_type, $end_x, $end_y, $allow_diagonal_lines) = @_ ;
@@ -364,12 +382,14 @@ else
 		'WIDTH' => 1,
 		'X_OFFSET' => $end_x,
 		'Y_OFFSET' => 1 ,
-		};
+		} ;
 	}
-	
+
 return($stripes, $width, $height) ;
 }
 
+#-----------------------------------------------------------------------------
+
 sub get_225_stripes
 {
 my ($position, $start, $body, $end) = @_ ;
@@ -383,8 +403,8 @@ push @stripes,
 	'WIDTH' => 1,
 	'X_OFFSET' => 0,
 	'Y_OFFSET' => 0,
-	};
-	
+	} ;
+
 for(my $xy = -$position - 1 ; $xy> 0 ; $xy--)
 	{
 	push @stripes,
@@ -394,9 +414,9 @@ for(my $xy = -$position - 1 ; $xy> 0 ; $xy--)
 		'WIDTH' => 1,
 		'X_OFFSET' => -$xy,
 		'Y_OFFSET' => $xy,
-		};
+		} ;
 	}
-	
+
 push @stripes,
 	{
 	'HEIGHT' => 1,
@@ -404,11 +424,13 @@ push @stripes,
 	'WIDTH' => 1,
 	'X_OFFSET' => $position ,
 	'Y_OFFSET' => -$position ,
-	};
-	
+	} ;
+
 return(@stripes) ;
 }
 
+#-----------------------------------------------------------------------------
+
 sub draw_right
 {
 my ($arrow_type, $end_x, $end_y) = @_ ;
@@ -423,11 +445,13 @@ push @{$stripes},
 	'WIDTH' => $width,
 	'X_OFFSET' => 0,
 	'Y_OFFSET' => 0,
-	};
-	
+	} ;
+
 return($stripes, $width, $height) ;
 }
 
+#-----------------------------------------------------------------------------
+
 sub draw_upright # or 45
 {
 my ($arrow_type, $end_x, $end_y, $allow_diagonal_lines) = @_ ;
@@ -456,12 +480,14 @@ else
 		'WIDTH' => $end_x,
 		'X_OFFSET' => 1,
 		'Y_OFFSET' => $end_y,
-		};
+		} ;
 	}
-	
+
 return($stripes, $width, $height) ;
 }
 
+#-----------------------------------------------------------------------------
+
 sub draw_rightup # or 45
 {
 my ($arrow_type, $end_x, $end_y, $allow_diagonal_lines) = @_ ;
@@ -490,12 +516,14 @@ else
 		'WIDTH' => 1,
 		'X_OFFSET' => $end_x,
 		'Y_OFFSET' => $end_y,
-		};
+		} ;
 	}
-	
+
 return($stripes, $width, $height) ;
 }
 
+#-----------------------------------------------------------------------------
+
 sub get_45_stripes
 {
 my ($position, $start, $body, $end) = @_ ;
@@ -509,8 +537,8 @@ push @stripes,
 	'WIDTH' => 1,
 	'X_OFFSET' => 0,
 	'Y_OFFSET' => 0,
-	};
-	
+	} ;
+
 for(my $xy = $position - 1 ; $xy > 0 ; $xy--)
 	{
 	push @stripes,
@@ -520,9 +548,9 @@ for(my $xy = $position - 1 ; $xy > 0 ; $xy--)
 		'WIDTH' => 1,
 		'X_OFFSET' => $xy,
 		'Y_OFFSET' => -$xy,
-		};
+		} ;
 	}
-	
+
 push @stripes,
 	{
 	'HEIGHT' => 1,
@@ -530,11 +558,13 @@ push @stripes,
 	'WIDTH' => 1,
 	'X_OFFSET' => $position ,
 	'Y_OFFSET' => -$position ,
-	};
-			
+	} ;
+
 return(@stripes) ;
 }
 
+#-----------------------------------------------------------------------------
+
 sub draw_downright # or 135
 {
 my ($arrow_type, $end_x, $end_y, $allow_diagonal_lines) = @_ ;
@@ -563,12 +593,14 @@ else
 		'WIDTH' => $width - 1,
 		'X_OFFSET' => 1,
 		'Y_OFFSET' => $end_y,
-		};
+		} ;
 	}
-	
+
 return($stripes, $width, $height) ;
 }
 
+#-----------------------------------------------------------------------------
+
 sub draw_rightdown # or 135
 {
 my ($arrow_type, $end_x, $end_y, $allow_diagonal_lines) = @_ ;
@@ -597,12 +629,14 @@ else
 		'WIDTH' => 1,
 		'X_OFFSET' => $end_x,
 		'Y_OFFSET' => 1,
-		};
+		} ;
 	}
-	
+
 return($stripes, $width, $height) ;
 }
 
+#-----------------------------------------------------------------------------
+
 sub get_135_stripes
 {
 my ($position, $start, $body, $end) = @_ ;
@@ -616,8 +650,8 @@ push @stripes,
 	'WIDTH' => 1,
 	'X_OFFSET' => 0 ,
 	'Y_OFFSET' => 0 ,
-	};
-	
+	} ;
+
 for(my $xy = 1 ; $xy < $position ; $xy++)
 	{
 	push @stripes,
@@ -627,9 +661,9 @@ for(my $xy = 1 ; $xy < $position ; $xy++)
 		'WIDTH' => 1,
 		'X_OFFSET' => $xy,
 		'Y_OFFSET' => $xy,
-		};
+		} ;
 	}
-	
+
 push @stripes,
 	{
 	'HEIGHT' => 1,
@@ -637,7 +671,7 @@ push @stripes,
 	'WIDTH' => 1,
 	'X_OFFSET' => $position,
 	'Y_OFFSET' => $position,
-	};
+	} ;
 
 return(@stripes) ;
 }
@@ -681,11 +715,11 @@ my ($self, $name) = @_ ;
 
 if($name eq 'start')
 	{
-	return( {X => 0, Y => 0, NAME => 'start'} ) ;
+	return( {X => 0, Y => 0, NAME => 'start', CHAR => 'X'} ) ;
 	}
 elsif($name eq 'end')
 	{
-	return( {X => $self->{END_X}, Y => $self->{END_Y}, NAME => 'end'} ) ;
+	return( {X => $self->{END_X}, Y => $self->{END_Y}, NAME => 'end', CHAR => 'Y'} ) ;
 	}
 else
 	{
@@ -695,9 +729,9 @@ else
 
 #-----------------------------------------------------------------------------
 
-sub get_section_direction
+sub get_direction
 {
-my ($self, $section_index) = @_ ;
+my ($self) = @_ ;
 
 return $self->{DIRECTION} ;
 }
@@ -721,7 +755,7 @@ elsif($connector_name eq 'end')
 	{
 	my ($x_offset, $y_offset, $width, $height, undef) = 
 		$self->resize(-1, -1, $self->{END_X} + $x_offset, $self->{END_Y} + $y_offset, $hint) ;
-
+	
 	return 
 		$x_offset, $y_offset, $width, $height,
 		{X => $self->{END_X}, Y => $self->{END_Y}, NAME => 'end'} ;
@@ -764,16 +798,16 @@ if($is_start)
 	my $new_end_y = $self->{END_Y} - $y_offset ;
 	
 	$self->setup($self->{ARROW_TYPE}, $new_end_x, $new_end_y, $hint || $self->{DIRECTION},$self ->{ALLOW_DIAGONAL_LINES}, $self->{EDITABLE}) ;
-
+	
 	return($x_offset, $y_offset, $self->{WIDTH}, $self->{HEIGHT}, 'start') ;
 	}
 else
 	{
 	my $new_end_x = $new_x ;
 	my $new_end_y = $new_y ;
-
+	
 	$self->setup($self->{ARROW_TYPE}, $new_end_x, $new_end_y, $hint || $self->{DIRECTION}, $self ->{ALLOW_DIAGONAL_LINES}, $self->{EDITABLE}) ;
-
+	
 	return(0, 0, $self->{WIDTH}, $self->{HEIGHT}, 'end') ;
 	}
 }
@@ -782,11 +816,11 @@ else
 
 sub edit
 {
-my ($self) = @_ ;
+my ($self, $asciio) = @_ ;
 
 return unless $self->{EDITABLE} ;
 
-display_arrow_edit_dialog($self->{ARROW_TYPE}) ; # inline modification
+$self->display_arrow_edit_dialog() ;
 
 my ($stripes, $width, $height, $x_offset, $y_offset) =
 	$direction_to_arrow{$self->{DIRECTION}}->($self->{ARROW_TYPE}, $self->{END_X}, $self->{END_Y}) ;
@@ -794,130 +828,6 @@ my ($stripes, $width, $height, $x_offset, $y_offset) =
 $self->set(STRIPES => $stripes,) ;
 }
 
-use Glib ':constants';
-use Gtk2 -init;
-use Glib qw(TRUE FALSE);
-
-sub display_arrow_edit_dialog
-{
-my ($rows) = @_ ;
-
-my $window = new Gtk2::Window() ;
-
-my $dialog = Gtk2::Dialog->new('Arrow attributes', $window, 'destroy-with-parent')  ;
-$dialog->set_default_size (450, 505);
-$dialog->add_button ('gtk-ok' => 'ok');
-
-#~ my $vbox = $dialog->vbox ;
-my $dialog_vbox = $dialog->vbox ;
-
-my $vbox = Gtk2::VBox->new (FALSE, 5);
-$dialog_vbox->pack_start ($vbox, TRUE, TRUE, 0);
-
-$vbox->pack_start (Gtk2::Label->new (""),
-		 FALSE, FALSE, 0);
-
-my $sw = Gtk2::ScrolledWindow->new;
-$sw->set_shadow_type ('etched-in');
-$sw->set_policy ('automatic', 'automatic');
-$vbox->pack_start ($sw, TRUE, TRUE, 0);
-
-# create model
-my $model = create_model ($rows);
-
-# create tree view
-my $treeview = Gtk2::TreeView->new_with_model ($model);
-$treeview->set_rules_hint (TRUE);
-$treeview->get_selection->set_mode ('single');
-
-add_columns($treeview, $rows);
-
-$sw->add($treeview);
-
-$treeview->show() ;
-$vbox->show() ;
-$sw->show() ;
-
-$dialog->run() ;
-
-$dialog->destroy ;
-}
-
-#-----------------------------------------------------------------------------
-
-sub create_model 
-{
-my ($rows) = @_ ;
-
-my $model = Gtk2::ListStore->new(qw/Glib::String Glib::String Glib::String  Glib::String  Glib::String Glib::String Glib::Boolean/);
-
-foreach my $row (@{$rows}) 
-	{
-	my $iter = $model->append;
-
-	my $column = 0 ;
-	$model->set ($iter, map {$column++, $_} @{$row}) ;
-	}
-
-return $model;
-}
-
-#-----------------------------------------------------------------------------
-
-sub add_columns 
-{
-my ($treeview, $rows) = @_ ;
-my $model = $treeview->get_model;
-
-# column for row titles
-my $row_renderer = Gtk2::CellRendererText->new;
-$row_renderer->set_data (column => 0);
-
-$treeview->insert_column_with_attributes
-			(
-			-1, '', $row_renderer,
-			text => 0,
-			) ;
-my $column = $treeview->get_column(0) ;
-$column->set_sizing('fixed') ;
-$column->set_fixed_width(120) ;
-
-my $current_column = 1 ;
-for my $column_title('start', 'body', 'connection', 'body_2', 'end')
-	{
-	my $renderer = Gtk2::CellRendererText->new;
-	$renderer->signal_connect (edited => \&cell_edited, [$model, $rows]);
-	$renderer->set_data (column => $current_column );
-
-	$treeview->insert_column_with_attributes 
-				(
-				-1, $column_title, $renderer,
-				text => $current_column,
-				editable => 6, 
-				);
-				
-	$current_column++ ;
-	}
-}
-
-#-----------------------------------------------------------------------------
-
-sub cell_edited 
-{
-my ($cell, $path_string, $new_text, $model_and_rows) = @_;
-
-my ($model, $rows) = @{$model_and_rows} ;
-
-my $path = Gtk2::TreePath->new_from_string ($path_string);
-my $column = $cell->get_data ("column");
-my $iter = $model->get_iter($path);
-my $row = ($path->get_indices)[0];
-
-$rows->[$row][$column] = $new_text ;
-
-$model->set($iter, $column, $new_text);
-}
-
 #-----------------------------------------------------------------------------
 
 1 ;
