@@ -1,4 +1,6 @@
 
+use File::Slurp ;
+
 register_import_export_handlers 
 	(
 	txt => 
@@ -20,11 +22,8 @@ if($self->{CREATE_BACKUP} && -e $file)
 	copy($file,"$file.bak") or die "export_pod: Copy failed while making backup copy: $!";		
 	}
 
-write_file($file, $self->transform_elements_to_ascii_buffer()) ;
-#~ open FH, ">:encoding(utf8)",   $file_name;
-#~ print FH $self->transform_elements_to_ascii_buffer() ;
-#~ close FH ;
+my $saved = write_file($file, {binmode => ':utf8'}, $self->transform_elements_to_ascii_buffer()) ;
 
-return ;
+return $saved ;
 }
 
